/*
 * Copyright (c) 2002, The Tendra Project <http://www.ten15.org/>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice unmodified, this list of conditions, and the following
 *    disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 *    		 Crown Copyright (c) 1997
 *
 *    This TenDRA(r) Computer Program is subject to Copyright
 *    owned by the United Kingdom Secretary of State for Defence
 *    acting through the Defence Evaluation and Research Agency
 *    (DERA).  It is made available to Recipients with a
 *    royalty-free licence for its use, reproduction, transfer
 *    to other parties and amendment for any purpose not excluding
 *    product development provided that any such use et cetera
 *    shall be deemed to be acceptance of the following conditions:-
 *
 *        (1) Its Recipients shall ensure that this Notice is
 *        reproduced upon any copies or amended versions of it;
 *
 *        (2) Any amended version of it shall be clearly marked to
 *        show both the nature of and the organisation responsible
 *        for the relevant amendment or amendments;
 *
 *        (3) Its onward transfer from a recipient to another
 *        party shall be deemed to be that party's acceptance of
 *        these conditions;
 *
 *        (4) DERA gives no warranty or assurance as to its
 *        quality or suitability for any purpose and DERA accepts
 *        no liability whatsoever in relation to any use to which
 *        it may be put.
 *
 * $TenDRA: tendra/src/tools/tnc/shape.h,v 1.5 2004/03/08 14:48:12 bp Exp $
 */


#ifndef SHAPE_INCLUDED
#define SHAPE_INCLUDED


/*
 *    BASIC SHAPE CONSTRUCTION ROUTINES
 */

extern node *sh_bottom;
extern node *sh_proc;
extern node *sh_top;

extern node *sh_bitfield(node *) ;
extern node *sh_compound(node *) ;
extern node *sh_floating(node *) ;
extern node *sh_integer(node *) ;
extern node *sh_nof(node *, node *) ;
extern node *sh_offset(node *, node *) ;
extern node *sh_pointer(node *) ;
extern void init_shapes(void) ;
extern node *string_length(node *) ;
extern node *copy_node(node *) ;


/*
 *    BASIC SHAPE CHECKING ROUTINES
 */

extern node *check_shapes(node *, node *, int) ;
extern node *lub(node *, node *) ;
extern node *check1(int, node *) ;
extern node *check2(int, node *, node *) ;
extern node *checkn(int, node *, int) ;
extern node *normalize(node *) ;


/*
 *    TOKEN EXPANSION ROUTINES
 */

extern node *expand_tok(node *) ;
extern node *expand_fully(node *) ;
extern node *set_token_args(construct **, node *, int) ;
extern void expand_all(void) ;


#endif
